/*  $Id: EventMonitor.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Timer;
import java.util.TimerTask;
/** 
 * Periodic event monitoring.
 * Periodic check of arrival of events between each timer top.
 * Here an event can be in fact anything. It 
 * is not direcly linked to the Event class.
 */
public class EventMonitor extends TimerTask implements 
                                Observable<EventMonitor> {
    /**
     * Create new monitor
     * @param period delay in milliseconds between two consecutive check.
     */
    public EventMonitor(long period) {
        _period=period;
        _timer=new Timer();
    }
    /** notify to this monitor that an event occurs */
    public void notifyEvent() {
        synchronized (this) {
            _cpt++;
        }
    }
    /**
     * Start monitoring.
     */
    public void start() {
        _timer.scheduleAtFixedRate(this,_period,_period);
    }
    /** 
     * Update monitor status.
     * TimerTask implementation. Checks events counter to compute monitor status.
     */
    public void run() {
        boolean test;
        synchronized (this) {
            test=_cpt!=0;
            _cpt=0;
        }
        if (test!=_active) {
            _active=test;
            _obsManager.notify(this);
        }
    }
    /** 
     * monitor status
     * @return 
     * - true when at least one notification has been recieved during last
     * period.
     * - false when no notification has been recieved.
     */ 
    public boolean isActive() {
        return _active;
    }
    /** 
     * Get the timer controlling the periodic checks. 
     * @return the timer.
     */ 
    public Timer getTimer() {
        return _timer;
    }
    /** Arrêter l'activité de d'observation et de comptage
     * des événements */
    public void dispose() {
        _timer.cancel();
    }
    /** Compteur d'evenement pour la période courante */
    private int _cpt=0;
    /** activity indicator. */
    private boolean _active=false;
    /** periodic check delay */
    private long _period;
    /** 
     * Timer controlling periodic checks.
     * @djin.relation composition
     */
    private Timer _timer;
    /** 
     * Observable implementation.
     * @djin.relation composition
     */
    private ObserverManager<EventMonitor> _obsManager=
                            new ObserverManager<EventMonitor>();
    // --- Observable implementation ---
    public ObserverManager<EventMonitor> getObserverManager() {
        return _obsManager;
    }
}
